from collections import OrderedDict
import nomenclate as nm
from . import basetest
from nomenclate.core.tools import (
    combine_dicts,
    gen_dict_key_matches,
    get_keys_containing
)


class TestCombineDicts(basetest.TestBase):
    def test_with_dict_with_nomenclate_object(self):
        self.assertDictEqual(combine_dicts({1: 1, 2: 2, 3: 3}, nm.Nom(name='test', purpose='lots').state),
                             {'decorator': '', 1: 1, 2: 2, 3: 3, 'name': 'test', 'type': '', 'side': '',
                              'childtype': '', 'var': '', 'location': '', 'purpose': 'lots'})

    def test_only_dicts(self):
        self.assertDictEqual(combine_dicts({1: 1, 2: 2, 3: 3}, {4: 4, 5: 5, 6: 6}),
                             {1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6})

    def test_no_dicts_or_kwargs(self):
        self.assertDictEqual(combine_dicts('five', 5, None, [], 'haha'),
                             {})

    def test_kwargs(self):
        self.assertDictEqual(combine_dicts(parse='test', plush=5),
                             {'parse': 'test', 'plush': 5})

    def test_dicts_and_kwargs(self):
        self.assertDictEqual(combine_dicts({1: 1, 2: 2, 3: 3}, {4: 4, 5: 5, 6: 6}, parse='test', plush=5),
                             {'parse': 'test', 'plush': 5, 1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6})

    def test_dicts_and_kwargs_with_dict_overlaps(self):
        self.assertDictEqual(combine_dicts({1: 1, 2: 2, 3: 3}, {2: 4, 4: 5, 5: 3}, parse='test', plush=5),
                             {'parse': 'test', 'plush': 5, 1: 1, 2: 4, 3: 3, 4: 5, 5: 3})

    def test_dicts_and_kwargs_and_ignorables(self):
        self.assertDictEqual(combine_dicts({1: 1, 2: 2, 3: 3}, 5, 'the', None, parse='test', plush=5),
                             {'parse': 'test', 'plush': 5, 1: 1, 2: 2, 3: 3})


class TestGenDictKeyMatches(basetest.TestBase):
    def test_with_nested(self):
        self.checkEqual(list(gen_dict_key_matches('name', {1: 1, 2: 2, 3: 3, 'test': {'name': 'mesh',
                                                                                      'test': {'name': 'bah'}},
                                                           'name': 'test', 'discipline': 'lots'})),
                        ['mesh', 'bah', 'test'])

    def test_not_nested(self):
        self.assertListEqual(list(gen_dict_key_matches('name',
                                                       {1: 1, 2: 2, 3: 3, 'name': 'mesh', 'discipline': 'lots'})),
                             ['mesh'])

    def test_list(self):
        self.checkEqual(list(gen_dict_key_matches('name',
                                                  {1: 1, 2: 2, 3: 3, 'test': {'name': 'mesh',
                                                                              'test': {'name': 'bah'},
                                                                              'suffixes': {'name':
                                                                                               {'bah': 'fah'}}},
                                                   'list': [{'name': 'nested_list'}, 5]})),
                        ['mesh', 'bah', {'bah': 'fah'}, 'nested_list'])

    def test_empty(self):
        self.assertListEqual(list(gen_dict_key_matches('name', {})),
                             [])

    def test_simple(self):
        self.assertEquals(next(gen_dict_key_matches('test', {'test': 1})), 1)

    def test_simple_with_path(self):
        self.assertEquals(next(gen_dict_key_matches('test', {'test': 1}, full_path=True)), (['test'], 1))

    def test_list_full_path(self):
        self.checkEqual(list(gen_dict_key_matches('name',
                                                  {1: 1, 2: 2, 3: 3, 'test': {'name': 'mesh',
                                                                              'test': {'name': 'bah'},
                                                                              'suffixes': {'name':
                                                                                               {'bah': 'fah'}}},
                                                   'list': [{'name': 'nested_list'}, 5]},
                                                  full_path=True)),
                        ['mesh', 'bah', {'bah': 'fah'}, 'nested_list'])

    def test_mock_config_find(self):
        self.checkEqual(
            list(get_keys_containing('naming_formats', OrderedDict([('overall_config', {'version_padding': 3}),
                                                                    ('options', {
                                                                        'discipline': {
                                                                            'animation': 'AN ANI ANIM ANIMN',
                                                                            'lighting': 'LT LGT LGHT LIGHT',
                                                                            'compositing': 'CM CMP COMP COMPG',
                                                                            'rigging': 'RG RIG RIGG RIGNG',
                                                                            'modeling': 'MD MOD MODL MODEL',
                                                                            'matchmove': 'MM MMV MMOV MMOVE'},
                                                                        'side': ['left', 'right', 'center']}),
                                                                    ('naming_formats', {
                                                                        'node': {
                                                                            'default': 'side_location_nameDecoratorVar_childtype_purpose_type',
                                                                            'format_archive': 'side_name_space_purpose_decorator_childtype_type',
                                                                            'format_lee': 'type_childtype_space_purpose_name_side'},
                                                                        'texturing': {
                                                                            'shader': 'side_name_type'}})]))),
            ['node', 'texturing'])


class TestGetKeysContaining(basetest.TestBase):
    def test_simple(self):
        self.assertEquals(get_keys_containing('test', {'test': 1}), 1)

    def test_not_existing(self):
        self.assertEquals(get_keys_containing('mest', {'test': 1}), None)

    def test_mock_config_find(self):
        self.checkEqual(
            list(get_keys_containing('naming_formats', OrderedDict([('overall_config', {'version_padding': 3}),
                                                                    ('options', {
                                                                        'discipline': {'animation': 'AN ANI ANIM ANIMN',
                                                                                       'lighting': 'LT LGT LGHT LIGHT',
                                                                                       'compositing': 'CM CMP COMP COMPG',
                                                                                       'rigging': 'RG RIG RIGG RIGNG',
                                                                                       'modeling': 'MD MOD MODL MODEL',
                                                                                       'matchmove': 'MM MMV MMOV MMOVE'},
                                                                        'side': ['left', 'right', 'center']}),
                                                                    ('naming_formats', {
                                                                        'node': {
                                                                            'default': 'side_location_nameDecoratorVar_childtype_purpose_type',
                                                                            'format_archive': 'side_name_space_purpose_decorator_childtype_type',
                                                                            'format_lee': 'type_childtype_space_purpose_name_side'},
                                                                        'texturing': {'shader': 'side_name_type'}})]))),
            ['node', 'texturing'])



from six import iteritems
import re
import nomenclate.core.rendering as rendering
import nomenclate.core.tokens as tokens
import nomenclate.core.errors as exceptions
import nomenclate.core.nomenclature as nom
from . import basetest


class TestNomenclativeBase(basetest.TestBase):
    def setUp(self):
        super(TestNomenclativeBase, self).setUp()
        self.nomenclative_valid = rendering.Nomenclative('side_location_nameDecoratorVar_childtype_purpose_type')
        self.nomenclative_valid_short = rendering.Nomenclative('side_name_type')
        self.nomenclative_invalid = rendering.Nomenclative('test_labelside')

        self.token_test_dict = {'side': 'left',
                                'location': 'rear',
                                'name': 'test',
                                'decorator': '',
                                'var': 'A',
                                'childtype': 'joints',
                                'purpose': 'offset',
                                'type': 'group'}

        self.fixtures.append([self.nomenclative_valid,
                              self.nomenclative_valid_short,
                              self.nomenclative_invalid,
                              self.token_test_dict])


class TestNomenclativeProcessMatches(TestNomenclativeBase):
    def test_valid(self):
        test_dict = self.token_test_dict.copy()
        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid.str)
        for token, value in iteritems(test_dict):
            self.nomenclative_valid.add_match(*value)
        self.assertEquals(self.nomenclative_valid.process_matches(),
                          'left_rear_testA_joints_offset_group')

    def test_valid_short(self):
        test_dict = self.token_test_dict.copy()
        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid_short.str)
        for token, value in iteritems(test_dict):
            if isinstance(value, str):
                pass
            else:
                self.nomenclative_valid_short.add_match(*value)
        self.assertEquals(self.nomenclative_valid_short.process_matches(),
                          'left_test_group')

    def test_invalid(self):
        self.assertEquals(self.nomenclative_invalid.process_matches(),
                          self.nomenclative_invalid.str)


class TestNomenclativeAddMatch(TestNomenclativeBase):
    def test_valid(self):
        test_dict = self.token_test_dict.copy()
        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid.str)
        for token, value in iteritems(test_dict):
            self.nomenclative_valid.add_match(*value)
        self.assertEquals(self.nomenclative_valid.process_matches(),
                          'left_rear_testA_joints_offset_group')

    def test_short_valid(self):
        test_dict = self.token_test_dict.copy()
        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid_short.str)
        for token, value in iteritems(test_dict):
            if not isinstance(value, str):
                self.nomenclative_valid_short.add_match(*value)
        self.assertEquals(self.nomenclative_valid_short.process_matches(),
                          'left_test_group')

    def test_overlap(self):
        test_dict = {'name': 'left', 'side': 'left'}
        test_overlap = {'side_name': 'overlapped'}
        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid_short.str)
        rendering.InputRenderer._prepend_token_match_objects(test_overlap, self.nomenclative_valid_short.str)

        for token, value in iteritems(test_dict):
            if not isinstance(value, str):
                self.nomenclative_valid_short.add_match(*value)

        for token, value in iteritems(test_overlap):
            if not isinstance(value, str):
                self.assertRaises(exceptions.OverlapError, self.nomenclative_valid_short.add_match, *value)

    def test_non_regex_match_object(self):
        self.assertEquals(self.nomenclative_invalid.process_matches(),
                          self.nomenclative_invalid.str)


class TokenMatchBase(basetest.TestBase):
    def setUp(self):
        super(TokenMatchBase, self).setUp()
        self.regex_custom_group_match = next(re.compile(r'(?P<look>test)').finditer('test'))

        test_re_matches = re.compile(r'(?P<token>te)').finditer('test')
        self.token_match_start = rendering.TokenMatch(next(test_re_matches), 'mx', group_name='token')

        test_re_matches = re.compile(r'(?P<token>es)').finditer('test')
        self.token_match_mid = rendering.TokenMatch(next(test_re_matches), 'lz', group_name='token')

        test_re_matches = re.compile(r'(?P<token>st)').finditer('test')
        self.token_match_end = rendering.TokenMatch(next(test_re_matches), 'fr', group_name='token')

        self.fixtures.extend([self.token_match_start,
                              self.token_match_mid,
                              self.token_match_end])


class TokenMatchInit(TokenMatchBase):
    def test_missing_group(self):
        self.assertRaises(IndexError, rendering.TokenMatch, self.regex_custom_group_match, 'marg')

    def test_group_custom(self):
        rendering.TokenMatch(self.regex_custom_group_match, 'marg', group_name='look')


class TokenMatchContains(TokenMatchBase):
    def test_does_not_contain(self):
        self.assertFalse(self.token_match_start in self.token_match_end)

    def test_contains(self):
        self.assertTrue(self.token_match_start in self.token_match_mid)

    def test_invalid_type(self):
        self.assertRaises(NotImplementedError, self.token_match_start.__contains__, 5)


class TokenMatchEquals(TokenMatchBase):
    def test_equal(self):
        self.assertTrue(self.token_match_start,
                        rendering.TokenMatch(next(re.compile(r'(?P<token>te)').finditer('test')), 'mx'))

    def test_inequal(self):
        self.assertTrue(self.token_match_start,
                        self.token_match_mid)

    def test_invalid_type(self):
        self.assertRaises(NotImplementedError, self.token_match_start.__eq__, 5)


class TokenMatchGT(TokenMatchBase):
    def test_equal(self):
        self.assertFalse(self.token_match_start >
                         rendering.TokenMatch(next(re.compile(r'(?P<token>te)').finditer('test')), 'mx'))

    def test_inequal(self):
        self.assertFalse(self.token_match_mid > self.token_match_start)

    def test_invalid_type(self):
        self.assertRaises(NotImplementedError, self.token_match_start.__gt__, 5)


class TokenMatchLT(TokenMatchBase):
    def test_equal(self):
        self.assertFalse(self.token_match_start <
                         rendering.TokenMatch(next(re.compile(r'(?P<token>te)').finditer('test')), 'mx'))

    def test_inequal(self):
        self.assertFalse(self.token_match_start < self.token_match_mid)

    def test_invalid_type(self):
        self.assertRaises(NotImplementedError, self.token_match_start.__lt__, 5)


class TokenMatchAdjustPosition(TokenMatchBase):
    def test_valid_adjust_alongside(self):
        test_re_matches = re.compile(r'(?P<token>st)').finditer('test')
        token_match_end = rendering.TokenMatch(next(test_re_matches), 'fl', group_name='token')
        orig_start = token_match_end.start
        token_match_end.adjust_position(self.token_match_start, adjust_by_sub_delta=False)
        self.assertEquals(token_match_end.start,
                          orig_start - self.token_match_start.span)

    def test_valid_adjust_end(self):
        test_re_matches = re.compile(r'(?P<token>er)').finditer('tester')
        token_match_end = rendering.TokenMatch(next(test_re_matches), 'fl', group_name='token')
        orig_start = token_match_end.start
        token_match_end.adjust_position(self.token_match_start, adjust_by_sub_delta=False)
        self.assertEquals(token_match_end.start,
                          orig_start - self.token_match_start.span)

    def test_valid_adjust_double(self):
        test_re_matches = re.compile(r'(?P<token>te)').finditer('lesterte')
        token_match_end = rendering.TokenMatch(next(test_re_matches), 'fl', group_name='token')
        orig_start = token_match_end.start
        token_match_end.adjust_position(self.token_match_start, adjust_by_sub_delta=False)
        self.assertEquals(token_match_end.start,
                          orig_start - self.token_match_start.span)

    def test_valid_adjust_no_move(self):
        self.assertRaises(IndexError, self.token_match_start.adjust_position, self.token_match_end)

    def test_invalid_adjust(self):
        self.assertRaises(exceptions.OverlapError, self.token_match_start.adjust_position, self.token_match_mid)

    def test_invalid_type(self):
        self.assertRaises(IOError, self.token_match_start.adjust_position, 5)


class TestInputRendererBase(TestNomenclativeBase):
    def setUp(self):
        super(TestInputRendererBase, self).setUp()
        self.ir = rendering.InputRenderer
        self.nom = nom.Nomenclate()
        self.nom.side.set('left')
        self.nom.name.set('testObject')
        self.nom.type.set('locator')
        self.nom.var.set('A')
        self.fixtures.append([self.ir, self.nom])


class TestInputRendererProcessTokenAugmentations(TestInputRendererBase):
    def test_from_nomenclate_upper(self):
        self.nom.side.case = 'upper'
        self.assertEquals(self.nom.get(),
                          'L_testObjectA_LOC')

    def test_from_nomenclate_lower(self):
        self.nom.side.case = 'lower'
        self.assertEquals(self.nom.get(),
                          'l_testObjectA_LOC')

    def test_from_upper(self):
        token_attr = tokens.TokenAttr('test', 'name')
        token_attr.case = 'upper'
        self.assertEquals(rendering.RenderBase.process_token_augmentations('test', token_attr),
                          'TEST')

    def test_from_lower(self):
        token_attr = tokens.TokenAttr('test', 'name')
        token_attr.case = 'lower'
        self.assertEquals(rendering.RenderBase.process_token_augmentations('Test', token_attr),
                          'test')

    def test_from_none(self):
        token_attr = tokens.TokenAttr('test', 'name')
        self.assertEquals(rendering.RenderBase.process_token_augmentations('Test', token_attr),
                          'Test')

    def test_from_prefix(self):
        token_attr = tokens.TokenAttr('test', 'name')
        token_attr.prefix = 'v'
        self.assertEquals(rendering.RenderBase.process_token_augmentations('Test', token_attr),
                          'vTest')

    def test_from_suffix(self):
        token_attr = tokens.TokenAttr('test', 'name')
        token_attr.suffix = '_r'
        self.assertEquals(rendering.RenderBase.process_token_augmentations('test', token_attr),
                          'test_r')


class TestInputRendererGetAlphanumericIndex(TestInputRendererBase):
    def test_get__get_alphanumeric_index_integer(self):
        self.assertEquals(self.ir._get_alphanumeric_index(0),
                          [0, 'int'])

    def test_get__get_alphanumeric_index_char_start(self):
        self.assertEquals(self.ir._get_alphanumeric_index('a'),
                          [0, 'char_lo'])

    def test_get__get_alphanumeric_index_char_end(self):
        self.assertEquals(self.ir._get_alphanumeric_index('z'),
                          [25, 'char_lo'])

    def test_get__get_alphanumeric_index_char_upper(self):
        self.assertEquals(self.ir._get_alphanumeric_index('B'),
                          [1, 'char_hi'])

    def test_get__get_alphanumeric_index_error(self):
        self.assertRaises(IOError, self.ir._get_alphanumeric_index, 'asdf')


class TestInputRendererCleanupFormattingString(TestInputRendererBase):
    def test_cleanup_format(self):
        self.assertEquals(self.ir.cleanup_formatted_string('test_name _messed __ up LOC'),
                          'test_name_messed_upLOC')


class TestInputRendererGetVariationId(TestInputRendererBase):
    def test_get_variation_id_normal(self):
        self.assertEquals(rendering.RenderVar._get_variation_id(0), 'a')

    def test_get_variation_id_negative(self):
        self.assertEquals(rendering.RenderVar._get_variation_id(-4), '')

    def test_get_variation_id_negative_one(self):
        self.assertEquals(rendering.RenderVar._get_variation_id(-1), '')

    def test_get_variation_id_double_upper(self):
        self.assertEquals(rendering.RenderVar._get_variation_id(1046, capital=True), 'ANG')

    def test_get_variation_id_double_lower(self):
        self.assertEquals(rendering.RenderVar._get_variation_id(1046, capital=False), 'ang')


class TestInputRendererRenderUniqueTokens(TestInputRendererBase):
    def test_all_replaced(self):
        test_values = {'var': 'A', 'type': 'locator', 'side': 'left', 'version': 5}
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals(test_values,
                          {'var': 'A', 'type': 'LOC', 'side': 'l', 'version': 'v005'})

    def test_some_replaced(self):
        test_values = {'var': 'A', 'type': 'locator', 'side': 'left', 'version': 5}
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals(test_values,
                          {'var': 'A', 'type': 'LOC', 'side': 'l', 'version': 'v005'})

    def test_default_renderer(self):
        test_values = {'var': 'A',
                       'type': 'locator',
                       'side': 'left',
                       'version': 5,
                       'john': 'six',
                       'purpose': 'hierarchy'}

        self.nom.format = self.nom.format + '_john'
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals(test_values,
                          {'var': 'A',
                           'type': 'LOC',
                           'side': 'l',
                           'version': 'v005',
                           'john': 'six',
                           'purpose': 'hrc'})

    def test_empty(self):
        test_values = {}
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals(test_values,
                          {})

    def test_none_replaced(self):
        test_values = {'name': 'test', 'blah': 'marg', 'not_me': 'haha', 'la': 5}
        self.nom.merge_dict(test_values)
        test_values_unchanged = test_values.copy()
        test_values_unchanged['la'] = '5'
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals(test_values, test_values_unchanged)



import nomenclate as nm
import nomenclate.core.tokens as tokens
import nomenclate.core.errors as exceptions
from . import basetest


class TestTokenAttrBase(basetest.TestBase):
    def setUp(self):
        super(TestTokenAttrBase, self).setUp()
        self.nomenclate = nm.Nom()
        self.token_attr_dict_handler = tokens.TokenAttrDictHandler(self.nomenclate)
        self.fixtures.append(self.token_attr_dict_handler, )


class TestGetTokenAttr(TestTokenAttrBase):
    def test_get_existing(self):
        self.assertEquals(self.token_attr_dict_handler.get_token_attr('name'), tokens.TokenAttr('', 'name'))

    def test_get_not_existing(self):
        self.assertRaises(AttributeError, self.token_attr_dict_handler.get_token_attr, 'no')

    def test_is_none(self):
        token_attr = self.token_attr_dict_handler.get_token_attr('name')
        setattr(self.token_attr_dict_handler, token_attr.token, None)
        print(getattr(self.token_attr_dict_handler, token_attr.token))
        self.assertRaises(exceptions.SourceError, self.token_attr_dict_handler.get_token_attr, 'name')
        setattr(self.token_attr_dict_handler, token_attr.token, token_attr)


class TestTokenAttrs(TestTokenAttrBase):
    def test_get(self):
        self.checkEqual(list(self.token_attr_dict_handler.token_attrs),
                        [tokens.TokenAttr('', 'Var'),
                         tokens.TokenAttr('', 'purpose'),
                         tokens.TokenAttr('', 'type'),
                         tokens.TokenAttr('', 'side'),
                         tokens.TokenAttr('', 'location'),
                         tokens.TokenAttr('', 'childtype'),
                         tokens.TokenAttr('', 'name'),
                         tokens.TokenAttr('', 'Decorator')])


class TestPurge(TestTokenAttrBase):
    def test_default(self):
        handler = tokens.TokenAttrDictHandler(self.nomenclate)
        handler.purge_tokens()
        self.assertEquals(list(handler.token_attrs), [])

    def test_input(self):
        handler = tokens.TokenAttrDictHandler(self.nomenclate)
        handler.purge_tokens([token.token for token in handler.token_attrs])
        self.assertEquals(list(handler.token_attrs), [])


class TestValidateNameInFormatOrder(TestTokenAttrBase):
    def test_not_existing(self):
        handler = tokens.TokenAttrDictHandler(self.nomenclate)
        self.assertRaises(exceptions.FormatError,
                          handler._validate_name_in_format_order,
                          'no',
                          self.nomenclate.format_order)

    def test_existing(self):
        handler = tokens.TokenAttrDictHandler(self.nomenclate)
        handler._validate_name_in_format_order('name', self.nomenclate.format_order)


class TestEq(TestTokenAttrBase):
    def test_not_equal(self):
        other_token_dict_handler = tokens.TokenAttrDictHandler(nm.Nom())
        other_token_dict_handler.set_token_attrs({'name': 'bob'})
        self.assertFalse(self.token_attr_dict_handler == other_token_dict_handler)

    def test_equal(self):
        self.assertTrue(self.token_attr_dict_handler == tokens.TokenAttrDictHandler(nm.Nom()))

    def test_not_instance(self):
        self.assertFalse(self.token_attr_dict_handler == 5)


class TestStr(TestTokenAttrBase):
    def test_str(self):
        tokens = ['name', 'decorator', 'purpose', 'childtype', 'type', 'location', 'side', 'var']
        for token in tokens:
            self.assertIn(token, str(self.token_attr_dict_handler))



from six import iteritems, assertCountEqual
import mock
import os
import json
from tempfile import mkstemp
from pyfakefs import fake_filesystem
from collections import OrderedDict
from os.path import expanduser

import nomenclate.core.configurator as config
import nomenclate.core.errors as exceptions
from . import basetest


class TestConfiguratorBase(basetest.TestBase):
    def setUp(self):
        super(TestConfiguratorBase, self).setUp()
        self.maxDiff = 1000
        self.mock_config = MockConfig()
        self.cfg = self.mock_config.parser
        self.fixtures = [self.cfg, self.mock_config]

        # test values
        self.format_title = self.mock_config.format_title
        self.format_subcategory = self.mock_config.format_subcategory
        self.default_format = self.mock_config.default_format
        self.format_test = self.mock_config.format_test
        self.discipline_path = self.mock_config.discipline_path
        self.discipline_subsets = self.mock_config.discipline_subsets
        self.discipline_data = self.mock_config.discipline_data


class TestValidateConfigFile(TestConfiguratorBase):
    @mock.patch('nomenclate.core.configurator.os.path.isfile')
    def test_valid_file_no_file(self, mock_isfile):
        mock_isfile.return_value = False
        self.assertRaises(IOError, self.cfg.validate_config_file, '/mock/config.yml')


class TestGet(TestConfiguratorBase):
    def test_get_default_as_string(self):
        self.assertEquals(self.cfg.get([self.format_title, 'node', self.default_format], return_type=str),
                          'side_location_nameDecoratorVar_childtype_purpose_type')

    def test_get_options(self):
        self.assertTrue(self.checkEqual(self.cfg.get([self.format_title], return_type=list),
                                        ['texturing', 'node']))

    def test_query_valid_entry(self):
        self.cfg.get(self.format_test)
        self.assertRaises(exceptions.ResourceNotFoundError, self.cfg.get, [self.format_title, 'submuerts'])
        self.assertRaises(exceptions.ResourceNotFoundError, self.cfg.get, ['faming_subsets', self.default_format])
        self.assertRaises(exceptions.ResourceNotFoundError, self.cfg.get, ['faming_subsets', 'dubsteps'])

    def test_get_section_ordered_dict(self):
        self.assertEquals(self.cfg.get(self.discipline_path, return_type=OrderedDict),
                          OrderedDict(sorted(iteritems(self.discipline_data), key=lambda x: x[0])))

    def test_get_section_ordered_dict_full_path(self):
        self.assertEquals(self.cfg.get(self.discipline_path, return_type=OrderedDict, preceding_depth=-1),
                          {'options': {
                              'discipline': OrderedDict(sorted(iteritems(self.discipline_data), key=lambda x: x[0]))}})

    def test_get_section_ordered_dict_partial_path(self):
        self.assertEquals(self.cfg.get(self.discipline_path, return_type=OrderedDict, preceding_depth=0),
                          {'discipline': OrderedDict(sorted(iteritems(self.discipline_data), key=lambda x: x[0]))})

    def test_get_disciplines_as_string(self):
        self.assertTrue(self.checkEqual(self.cfg.get(self.discipline_path, return_type=str).split(),
                                        self.discipline_subsets))

    def test_get_as_string_search(self):
        self.assertTrue(self.checkEqual(self.cfg.get('animation', return_type=str),
                                        'AN ANI ANIM ANIMN'))

    def test_get_as_dict(self):
        self.assertEquals(self.cfg.get(self.discipline_path, return_type=list, preceding_depth=-1),
                          {self.discipline_path[0]: {self.discipline_path[1]: self.discipline_subsets}})

    def test_get_as_dict_subdict(self):
        self.assertEquals(self.cfg.get(self.discipline_path, return_type=dict),
                          self.discipline_data)

    def test_get(self):
        self.assertTrue(self.checkEqual(self.cfg.get(self.discipline_path, return_type=dict), self.discipline_subsets))

    def test_list_sections(self):
        assertCountEqual(self,
                         self.cfg.get([], return_type=list),
                         ['overall_config', 'options', 'naming_formats'])

    def test_list_section_options(self):
        self.assertEquals(self.cfg.get(self.format_title, return_type=list),
                          ['node', 'texturing'])

    def test_default_get(self):
        self.assertEquals(self.cfg.get(return_type=str),
                          "")

    def test_default_get_no_return_type(self):
        self.assertEquals(self.cfg.get(),
                          ['overall_config', 'options', 'naming_formats'])


class TestGetDefaultConfigFile(TestConfiguratorBase):
    def test_existing(self):
        config.ConfigParse()

    def test_custom(self):
        fd, temp_path = mkstemp()
        f = open(temp_path, 'w')
        f.write(json.dumps({'name': 'john', 'location': 'top'}))
        f.close()
        custom_config = config.ConfigParse(temp_path)
        self.assertDictEqual(custom_config.config_file_contents, OrderedDict([('name', 'john'), ('location', 'top')]))
        os.close(fd)
        os.remove(temp_path)

    def test_custom_empty(self):
        fd, temp_path = mkstemp()
        self.assertRaises(IOError, config.ConfigParse.validate_config_file, temp_path)
        os.close(fd)
        os.remove(temp_path)

    def test_custom_no_yaml_data(self):
        fd, temp_path = mkstemp()
        f = open(temp_path, 'w')
        f.write('#Empty YAML File')
        f.close()
        self.assertRaises(IOError, config.ConfigParse.validate_config_file, temp_path)
        os.close(fd)
        os.remove(temp_path)

    @mock.patch('nomenclate.core.configurator.ConfigParse.validate_config_file')
    def test_no_valid_config_file(self, mock_validate_config_file):
        mock_validate_config_file.side_effect = IOError('mock: config file not found')
        self.assertRaises(exceptions.SourceError, config.ConfigParse)


class TestGetHandler(TestConfiguratorBase):
    def test_existing(self):
        config.ConfigEntryFormatter.get_handler(str, list)

    def test_not_existing(self):
        self.assertRaises(IndexError, config.ConfigEntryFormatter.get_handler, int, str)


class MockConfig(object):
    test_data = ('overall_config:\n'
                 '  version_padding: 3\n'
                 'naming_formats:\n'
                 '  node:\n'
                 '    default: side_location_nameDecoratorVar_childtype_purpose_type\n'
                 '    format_archive: side_name_space_purpose_decorator_childtype_type\n'
                 '    format_lee: type_childtype_space_purpose_name_side\n'
                 '  texturing:\n'
                 '    shader: side_name_type\n'
                 'options:\n'
                 '  discipline:\n'
                 '    animation: AN ANI ANIM ANIMN\n'
                 '    lighting: LT LGT LGHT LIGHT\n'
                 '    rigging: RG RIG RIGG RIGNG\n'
                 '    matchmove: MM MMV MMOV MMOVE\n'
                 '    compositing: CM CMP COMP COMPG\n'
                 '    modeling: MD MOD MODL MODEL\n'
                 '  side:\n'
                 '    - left\n'
                 '    - right\n'
                 '    - center\n')

    def __init__(self):
        self.build_test_config()

    @mock.patch('nomenclate.core.configurator.os.path.getsize')
    @mock.patch('nomenclate.core.configurator.os.path.isfile')
    @mock.patch('nomenclate.core.configurator.open', mock.mock_open(read_data=test_data))
    def build_test_config(self, mock_isfile, mock_getsize):
        mock_isfile.return_value = True
        mock_getsize.return_value = 700
        self.parser = config.ConfigParse()
        self.fakefs = fake_filesystem.FakeFilesystem()

        self.fake_file_path = '/var/env/foobar.yml'
        self.fake_file = self.fakefs.CreateFile(self.fake_file_path, contents=self.test_data)
        fake_filesystem.FakeFile(self.fakefs)
        self.parser.rebuild_config_cache(self.fake_file_path)

        # test values
        self.format_title = 'naming_formats'
        self.format_subcategory = 'node'
        self.default_format = 'default'
        self.format_test = [self.format_title, self.format_subcategory, self.default_format]
        self.discipline_path = ['options', 'discipline']
        self.discipline_subsets = ['animation', 'modeling', 'rigging', 'compositing', 'matchmove', 'lighting']
        self.discipline_data = {'animation': 'AN ANI ANIM ANIMN',
                                'compositing': 'CM CMP COMP COMPG',
                                'lighting': 'LT LGT LGHT LIGHT',
                                'matchmove': 'MM MMV MMOV MMOVE',
                                'modeling': 'MD MOD MODL MODEL',
                                'rigging': 'RG RIG RIGG RIGNG'}


class TestFormatters(TestConfiguratorBase):
    def test_base_formatter_init(self):
        self.assertRaises(NotImplementedError, config.BaseFormatter.format_result, '')

    def test_dict_to_ordered_dict(self):
        self.assertEquals(config.DictToOrderedDict.format_result({}),
                          OrderedDict())

    def test_none_to_dict(self):
        self.assertEquals(config.NoneToDict.format_result(None),
                          {})

    def test_int_to_list(self):
        self.assertEquals(config.IntToList.format_result(1),
                          [1])

    def test_list_to_str(self):
        self.assertEquals(config.ListToString.format_result(['john', 'kate']),
                          'john kate')



#!/usr/bin/env python
from six import iteritems
from . import errors as exceptions
import nomenclate.settings as settings

MODULE_LEVEL_OVERRIDE = None


class TokenAttr(object):
    """ A TokenAttr represents a string token that we want to replace in a given nomenclate.core.formatter.FormatString
        It has 3 augmentation properties:

            TokenAttr().case
            TokenAttr().prefix
            TokenAttr().suffix

        These three settings enforce that after final rendering (finding matches in the config for the current
        token's label and any custom rendering syntax the user added) we will upper case the result and add
        either the given prefix or suffix no matter what.

    """
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LEVEL_OVERRIDE)

    def __init__(self, label=None, token=None):
        """

        :param label:
        :param token:
        """
        try:
            self.validate_entries(label)
        except exceptions.ValidationError:
            label = None
        self.validate_entries(token)
        self.raw_string = label if label is not None else ""
        self.raw_token = token
        self.case_setting = ""
        self.prefix_setting = ""
        self.suffix_setting = ""

    @property
    def token(self):
        """ Get or set the current token. Setting the token to a new value means it will be validated and then
            added as the internal "raw_token" which will be used to look up any given config value or be used if
            no config value is found.

        """
        return self.raw_token.lower()

    @token.setter
    def token(self, token):
        self.validate_entries(token)
        self.raw_token = token

    @property
    def label(self):
        """ Get or set the current token's "label" or value. Setting the label to a new value means it will be added
            as the internal "raw_string" which will be used to look up any given config value or be used if no config
            value is found.

        """
        return self.raw_string

    @label.setter
    def label(self, label):
        self.validate_entries(label)
        self.LOG.debug('Setting token attr %s -> %r' % (str(self), label))
        self.raw_string = label
        self.LOG.debug('%r and the raw_string is %s' % (self, self.raw_string))

    @property
    def case(self):
        """ Get or set the current TokenAttr's case. Setting the case to either 'upper' or 'lower' means it will be validated and then
            added as the internal "raw_token" which will be used to look up any given config value or be used if
            no config value is found.

        """
        return self.case_setting

    @case.setter
    def case(self, case):
        if case in ['upper', 'lower']:
            self.case_setting = case

    @property
    def prefix(self):
        return self.prefix_setting

    @prefix.setter
    def prefix(self, prefix):
        if isinstance(prefix, str):
            self.prefix_setting = prefix

    @property
    def suffix(self):
        return self.suffix_setting

    @suffix.setter
    def suffix(self, suffix):
        if isinstance(suffix, str):
            self.suffix_setting = suffix

    def set(self, value):
        self.label = value

    @staticmethod
    def validate_entries(*entries):
        for entry in entries:
            if isinstance(entry, (str, int)) or entry is None:
                continue
            else:
                raise exceptions.ValidationError('Invalid type %s, expected %s' % (type(entry), str))

    def __eq__(self, other):
        if isinstance(other, type(self)):
            return self.token == other.token and self.label == other.label
        else:
            return False

    def __ne__(self, other):
        return ((self.token, self.label) != (other.token, other.label))

    def __lt__(self, other):
        return ((self.token, self.label) < (other.token, other.label))

    def __le__(self, other):
        return ((self.token, self.label) <= (other.token, other.label))

    def __gt__(self, other):
        return ((self.token, self.label) > (other.token, other.label))

    def __ge__(self, other):
        return ((self.token, self.label) >= (other.token, other.label))
    
    def __str__(self):
        return str(self.label)

    def __repr__(self):
        return '<%s %s(%s):%r>' % (self.__class__.__name__, self.token, self.raw_token, self.label)



class TokenAttrDictHandler(object):
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LEVEL_OVERRIDE)

    def __init__(self, nomenclate_object):
        self.nom = nomenclate_object
        self.LOG.info('Initializing TokenAttrDictHandler with default values %s' % self.empty_state)
        self.set_token_attrs(self.empty_state)

    @property
    def tokens(self):
        return [token.token for token in self.token_attrs]

    @property
    def token_attrs(self):
        return self.gen_object_token_attributes(self)

    @property
    def state(self):
        return dict((name_attr.token, name_attr.label) for name_attr in self.token_attrs)

    @state.setter
    def state(self, input_dict):
        self.set_token_attrs(input_dict)

    @property
    def empty_state(self):
        self.LOG.info("Generating empty initial state from format order: %s" % self.nom.format_order)
        return {token: "" for token in self.nom.format_order}

    @property
    def unset_token_attrs(self):
        return [token_attr for token_attr in self.token_attrs if token_attr.label == '']

    @property
    def token_attr_dict(self):
        return dict([(attr.token, attr.label) for attr in self.token_attrs])

    def reset(self):
        for token_attr in self.token_attrs:
            token_attr.set('')

    def set_token_attrs(self, input_dict):
        if not input_dict:
            self.LOG.info('No changes to make, ignoring...')
            return

        self.LOG.info('Setting token attributes %s against current state %s' % (input_dict, self.state))

        for input_attr_name, input_attr_value in iteritems(input_dict):
            self.set_token_attr(input_attr_name, input_attr_value)

        self.LOG.info('Finished setting attributes on TokenDict %s' % [attr for attr in list(self.token_attrs)])

    def set_token_attr(self, token, value):
        self.LOG.info('set_token_attr() - Setting TokenAttr %s with value %r' % (token, value))
        token_attrs = list(self.token_attrs)

        if token not in [token_attr.token for token_attr in token_attrs]:
            self.LOG.warning('Token did not exist, creating %r=%r...' % (token, value))
            self._create_token_attr(token, value)
        else:
            for token_attr in token_attrs:
                self.LOG.debug('Checking if token %r is equal to preexisting token %r' % (token, token_attr.token))
                if token == token_attr.token:
                    self.LOG.debug('Found matching token, setting value to %r' % value)
                    token_attr.label = value
                    break

        if hasattr(self.nom, 'token_dict'):
            token_attr_instance = getattr(self, token.lower())
            self.LOG.info('Passing token attr %r to the nomenclate object instance for updating' % token_attr_instance)
            self.nom.notifier.notify_observer(token.lower(), token_attr_instance)

    def get_token_attr(self, token):
        token_attr = getattr(self, token.lower())
        if token_attr is None:
            msg = 'Instance has no %s token attribute set.' % token
            self.LOG.warn(msg)
            raise exceptions.SourceError(msg)
        else:
            return token_attr

    def _create_token_attr(self, token, value):
        self.LOG.debug('_create_token_attr(%s:%s)' % (token, repr(value)))
        self.__dict__[token.lower()] = TokenAttr(label=value, token=token)

    def purge_tokens(self, token_attrs=None):
        """ Removes tokens not found in the format order
        """
        if token_attrs is None:
            token_attrs = self.tokens
        self.LOG.info('Starting purge for target tokens %s' % token_attrs)
        for token_attr in [_ for _ in token_attrs if _ in self.tokens]:
            self.LOG.info('Deleting TokenAttr %s' % token_attr)
            delattr(self, token_attr)

    @staticmethod
    def gen_object_token_attributes(obj):
        for name, value in iteritems(obj.__dict__):
            if isinstance(value, TokenAttr):
                yield value

    @staticmethod
    def _validate_name_in_format_order(name, format_order):
        """ For quick checking if a key token is part of the format order
        """
        if name not in format_order:
            raise exceptions.FormatError('The name token %s is not found in the current format ordering' %
                                         format_order)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return all(map(lambda x: x[0] == x[1], zip(sorted(self.token_attrs, key=lambda x: x.token),
                                                       sorted(other.token_attrs, key=lambda x: x.token))))
        return False

    def __getattr__(self, name):
        try:
            return self.__dict__[name]
        except KeyError:
            object.__getattribute__(self.__dict__, name)

    def __str__(self):
        return ' '.join(['%s:%r' % (token_attr.token, token_attr.label) for token_attr in self.token_attrs])

    def __repr__(self):
        return '<%s %s>' % (self.__class__.__name__,
                            ' '.join(['%s:%s' % (_.token, _.label) for _ in self.token_attrs]))



import nomenclate.core.nomenclature as nom
from . import basetest


class TestCreation(basetest.TestBase):
    def test_initialize_with_dict_only_one(self):
        n = nom.Nomenclate({'name': 'test'})
        n.new = 'default'
        self.assertEquals(n.get(), 'test')

    def test_initialize_with_dict_only_end_and_start(self):
        n = nom.Nomenclate({'side': 'left', 'type': 'locator'})
        n.new = 'default'
        self.assertEquals(n.get(), 'l_LOC')

    def test_initialize_with_dict_incomplete_and_swap_format_from_new_string(self):
        n = nom.Nomenclate({'name': 'test', 'type': 'locator', 'var': 'A', 'side': 'left'})
        self.assertEquals(n.get(), 'l_testA_LOC')
        n.format = 'new_nameDecoratorVar_childtype_purpose_type_side'
        n.new = 'default'
        self.assertEquals(n.get(), 'default_testA_LOC_l')
        self.fixtures.append(n)

    def test_initialize_with_dict_incomplete_and_swap_format_from_path(self):
        n = nom.Nomenclate({'name': 'test', 'type': 'locator', 'var': 'A', 'side': 'left'})
        self.assertEquals(n.get(), 'l_testA_LOC')
        n.format = ['naming_formats', 'node', 'format_archive']
        self.assertEquals(n.get(), 'l_test_LOC')
        self.fixtures.append(n)

    def test_initialize_with_dict_complete(self):
        n = nom.Nomenclate({'name': 'test',
                            'decorator': 'J',
                            'childtype': 'joint',
                            'purpose': 'offset',
                            'var': 'A',
                            'side': 'left',
                            'type': 'locator',
                            'location': 'rear'})
        self.assertEquals(n.get(), 'l_rr_testJA_joint_offset_LOC')

    def test_initialize_with_attributes_incomplete(self):
        n = nom.Nomenclate({'name': 'test', 'type': 'locator', 'var': 'A', 'side': 'left', })
        self.assertEquals(n.get(), 'l_testA_LOC')
        self.fixtures.append(n)

    def test_initialize_with_attributes_complete(self):
        n = nom.Nomenclate()
        n.name = 'test'
        n.decorator = 'J'
        n.childtype = 'joint'
        n.purpose = 'offset'
        n.var = 'A'
        n.side = 'left'
        n.type = 'locator'
        n.location = 'rear'
        n.LOG.info(str(n.state))
        self.assertEquals(n.get(), 'l_rr_testJA_joint_offset_LOC')
        self.fixtures.append(n)

    def test_initialize_from_nomenclate_object(self):
        n_initial = nom.Nomenclate({'name': 'test', 'type': 'locator', 'var': 'A', 'side': 'left', })
        n_secondary = nom.Nomenclate(n_initial)
        self.assertEquals(n_secondary.get(), 'l_testA_LOC')
        self.fixtures.extend([n_secondary, n_initial])

    def test_initialize_from_nomenclate_state(self):
        n_initial = nom.Nomenclate({'name': 'test', 'type': 'locator', 'var': 'A', 'side': 'left', })
        state_dict = n_initial.state
        n_secondary = nom.Nomenclate(state_dict)
        self.assertEquals(n_secondary.get(), 'l_testA_LOC')
        self.fixtures.extend([n_secondary, n_initial, state_dict])

    def test_initialize_from_nomenclate_object_and_kwargs(self):
        n_initial = nom.Nomenclate({'name': 'test', 'type': 'locator', 'var': 'A', 'side': 'left', })
        n_secondary = nom.Nomenclate(n_initial, **{'name': 'blah', 'location': 'rear'})
        self.assertEquals(n_secondary.get(), 'l_rr_blahA_LOC')
        self.fixtures.extend([n_secondary, n_initial])

    def test_initialize_from_nomenclate_and_kwargs(self):
        n_initial = nom.Nomenclate({'name': 'test', 'type': 'locator', 'var': 'A', 'side': 'left', })
        n_secondary = nom.Nomenclate(n_initial, name='blah', location='rear')
        self.assertEquals(n_secondary.get(), 'l_rr_blahA_LOC')
        self.fixtures.extend([n_secondary, n_initial])

    def test_initialize_and_switch_format_then_set_properties(self):
        n = nom.Nomenclate({'name': 'test', 'type': 'locator', 'var': 'A', 'side': 'left'})
        n.initialize_format_options('new_nameDecoratorVar_childtype_purpose_type_side')
        n.name = 'default'
        self.assertEquals(n.get(), 'defaultA_LOC_l')

    def test_initialize_with_nomenclate_and_get_kwargs(self):
        n_initial = nom.Nomenclate({'name': 'test', 'type': 'locator', 'var': 'A', 'side': 'left'})
        n_secondary = nom.Nomenclate(n_initial)
        self.assertEquals(n_secondary.get(name='blah', location='rear'), 'l_rr_blahA_LOC')
        self.fixtures.extend([n_secondary, n_initial])


class TestAcceptanceMaya(basetest.TestBase):
    pass


class TestAcceptanceNamingFiletypes(basetest.TestBase):
    def test_saving_maya_file(self):
        n = nom.Nomenclate(name='SH010', var='A', ext='mb', initials='aw', discipline='animation', version=5)
        n.initialize_format_options('working_file')
        self.assertEquals(n.format, 'name_discipline_lodDecoratorVar_version_initials.ext')
        self.assertEquals(n.get(), 'SH010_ANIM_A_v005_aw.mb')
        self.fixtures.append(n)

    def test_saving_movie_file(self):
        n = nom.Nomenclate()
        n.initialize_format_options('techops_file')
        n.merge_dict({'shot': 'LSC_sh01',
                      'version': 8,
                      'name': 'Nesquick',
                      'type': 'SFX_MS',
                      'status': 'WIP',
                      'date': 'September 21, 2005',
                      'filetype': 'Quicktime',
                      'var': 'A',
                      'ext': 'mov',
                      'initials': 'aw',
                      'discipline': 'animation',
                      'quality': '540p',
                      'version1': 3,
                      'version_padding': 1,
                      'version1_format': 'v#',
                      'version1_padding': 1})
        self.assertEquals('LSC_sh01_v8_Nesquick_SFX_MS_WIP_v3_2005-09-21-540p_Quicktime.mov', n.get())
        n.date_format = '%m%d%y'
        self.assertEquals('LSC_sh01_v8_Nesquick_SFX_MS_WIP_v3_092105-540p_Quicktime.mov', n.get())

        self.fixtures.append(n)


class TestAcceptanceParsingExisting(basetest.TestBase):
    def test_normal_maya_node(self):
        pass

    def test_working_file(self):
        pass

    def test_asset_file(self):
        pass

    def test_movie_file(self):
        pass



# Mimicking these exception styles found here:
# https://github.com/frictionlessdata/tabulator-py/blob/master/tabulator/exceptions.py

# Module API


class NomenclateException(Exception):
    """Base Tabulator exception.
    """
    pass


class SourceError(NomenclateException):
    """Stream error.
    """
    pass


class SchemeError(NomenclateException):
    """Scheme error.
    """
    pass


class FormatError(NomenclateException):
    """Format error.
    """
    pass


class OptionsError(NomenclateException):
    """Options error.
    """
    pass


class ValidationError(NomenclateException):
    """IO error.
    """
    pass


class ResourceNotFoundError(NomenclateException):
    """HTTP error.
    """
    pass


class ResetError(NomenclateException):
    """Reset error.
    """
    pass


class OverlapError(NomenclateException):
    """Overlap error.
    """
    pass



#!/usr/bin/env python
""" This module does name parsing to recover information about the name based on common conventions.
    it will be very prone to missing information since names will be very variable, but hopefully
    it will be more intelligent as time goes on
"""
# Ensure Python 2/3 compatibility: http://python-future.org/compatible_idioms.html
from six import iteritems
import re
import datetime
import itertools
from . import configurator as config


class NameParser(object):
    """ This parses names of assets.  It assumes the usual convention of strings separated by underscores.

    """
    CONFIG_SIDES = config.ConfigParse().get(['options', 'side'])
    CONFIG_DISCIPLINES = config.ConfigParse().get(['options', 'discipline'])
    PARSABLE = ['basename', 'version', 'date', 'side', 'udim']

    REGEX_BASENAME = r'(?:^[-._]+)?([a-zA-Z0-9_\-|]+?)(?=[-._]{2,}|\.)'
    REGEX_SEPARATORS = r'[ ,_!?:\.\-]'

    REGEX_ABBR_ISLAND = r'(?:[{SEP}]+)({ABBR})(?:[{SEP}]+)'
    REGEX_ABBR_SEOS = r'(?:^|[a-z]|{SEP})({ABBR})(?:$|[A-Z][a-z]+|{SEP})'
    REGEX_ABBR_CAMEL = r'(?:[a-z])({ABBR})(?:$|[A-Z]|{SEP})'

    REGEX_VERSION = r'(?:^|[a-z]{{2,}}[a-uw-z]+|{SEP}[a-z]?[a-uw-z]+|{SEP})([vV]?[0-9]{{1,4}})(?={SEP}|$)'
    REGEX_UDIM = r'(?:[a-zA-Z]|[._-])(1[0-9]{3})(?:[._-])'
    REGEX_DATE = r'(?<!\d)(%s)(?!\d)'
    REGEX_CAMEL = r'(?:{SEP}?)((?<=[a-z])(?=[A-Z])|(?<=[A-Z])(?=[A-Z][a-z]))(?:{SEP}?)'

    @classmethod
    def parse_name(cls, name):
        """ Parses a name into a dictionary of identified subsections with accompanying information to
            correctly identify and replace if necessary

        :param name: str, string to be parsed
        :return: dict, dictionary with relevant parsed information
        """
        parse_dict = dict.fromkeys(cls.PARSABLE, None)
        parse_dict['date'] = cls.get_date(name)
        parse_dict['version'] = cls.get_version(name)
        parse_dict['udim'] = cls.get_udim(name)
        parse_dict['side'] = cls.get_side(name)
        parse_dict['basename'] = cls.get_base_naive(cls._reduce_name(name, parse_dict))
        return parse_dict

    @classmethod
    def get_side(cls, name, ignore=''):
        """ Checks a string for a possible side string token, this assumes its on its own
            and is not part of or camel cased and combined with a word.  Returns first found side to reduce duplicates.
            We can be safe to assume the abbreviation for the side does not have camel casing within its own word.

        :param name: str, string that represents a possible name of an object
        :param name: str, string that represents a possible name of an object
        :return: (None, str), either the found permutation of the side found in name or None
        """
        for side in cls.CONFIG_SIDES:
            """ Tried using a regex, however it would've taken too long to debug
            side_regex = cls._build_abbreviation_regex(side)
            result = cls._generic_search(name, side_regex, metadata={'side': side}, ignore=ignore)
            if result:
                return result
            """
            for permutations in cls.get_string_camel_patterns(side):
                for permutation in permutations:
                    result = cls._generic_search(name, permutation, metadata={'side': side}, ignore=ignore)
                    if result:
                        return result
        return None

    @classmethod
    def get_discipline(cls, name, ignore='', min_length=3):
        """ Checks a string for a possible discipline string token, this assumes its on its own
            and is not part of or camel cased and combined with a word.  Returns first found match to reduce duplicates.
            We can be safe to assume the abbreviation for the discipline does not have camel casing within its own word.

        :param name: str, the string based object name
        :param ignore: str, specific ignore string for the search to avoid
        :param min_length: int, minimum length for possible abbreviations of disciplines. Lower = more wrong guesses.
        :return: dict, match dictionary
        """
        for discipline in cls.CONFIG_DISCIPLINES:
            re_abbr = '({RECURSE}(?=[0-9]|[A-Z]|{SEPARATORS}))'.format(
                RECURSE=cls._build_abbreviation_regex(discipline),
                SEPARATORS=cls.REGEX_SEPARATORS)
            matches = cls._get_regex_search(name, re_abbr, ignore=ignore)
            if matches:
                matches = [m for m in matches if
                           re.findall('([a-z]{%d,})' % min_length, m['match'], flags=re.IGNORECASE)]
                if matches:
                    return matches[-1]
        return None

    @classmethod
    def get_base(cls, name):
        """ Checks a string for a possible base name of an object (no prefix, no suffix).
            We need to do a full parse to make sure we've ruled out all the possible other parse queries

        :param name: str, string that represents a possible name of an object
        :return: str, the detected basename
        """
        return cls.parse_name(name).get('basename', None)

    @classmethod
    def get_base_naive(cls, name, ignore=''):
        """ Checks a string for a possible base name of an object (no prefix, no suffix).
            We need to do a full parse to make sure we've ruled out all the possible other parse queries

        :param name: str, string that represents a possible name of an object
        :return: str, the detected basename
        """
        return cls._get_regex_search(name, cls.REGEX_BASENAME, match_index=0, ignore=ignore)

    @classmethod
    def get_version(cls, name):
        """ Checks a string for a possible version of an object (no prefix, no suffix).
            Assumes only up to 4 digit padding

        :param name: str, string that represents a possible name of an object
        :return: (float, int, list(str), None), gets the version number then the string matches
        """
        # Dates can confuse th
        # is stuff, so we'll check for that first and remove it from the string if found
        try:
            date = cls.get_date(name)
            date = date['datetime'].strftime(date['format'])
        except TypeError:
            pass
        return cls.get_version_naive(name, ignore=date or '')

    @classmethod
    def get_version_naive(cls, name, ignore=''):
        """ Checks a string for a possible version of an object (no prefix, no suffix) without filtering date out
            Assumes only up to 4 digit padding

        :param name: str, string that represents a possible name of an object
        :return: (float, int, list(str), None), gets the version number then the string matches
        """
        match = cls._get_regex_search(name, cls.REGEX_VERSION.format(SEP=cls.REGEX_SEPARATORS), ignore=ignore)

        if match is not None:
            if len(match) > 1:
                for m in match:
                    m.update({'version': int(m['match'].upper().replace('V', ''))})
                compound_version = '.'.join([str(m['version']) for m in match])
                compound_version = float(compound_version) if compound_version.count('.') == 1 else compound_version
                return {'compound_matches': match,
                        'compound_version': compound_version,
                        'pattern': match[0]['pattern'],
                        'input': match[0]['input']}

            elif len(match) == 1:
                match = match[0]
                match.update({'version': int(match['match'].upper().replace('V', ''))})
                return match

        return None

    @classmethod
    def get_udim(cls, name):
        """ Checks a string for a possible base name of an object (no prefix, no suffix)

        :param name: str, string that represents a possible name of an object
        :returns: int, the last found match because convention keeps UDIM markers at the end.
        """
        match = cls._get_regex_search(name, cls.REGEX_UDIM, match_index=-1)
        if match:
            match.update({'match_int': int(match['match'])})
            return match
        return None

    @classmethod
    def get_short(cls, name):
        """ Returns the short name of a Maya asset name or path

        :param name: str, string that represents a possible name of an object
        :return: str, the short name of the maya node
        """
        return name.split('|')[-1].split('//')[-1]

    @classmethod
    def get_date(cls, name):
        """ Checks a string for a possible date formatted into the name.  It assumes dates do not have other
            numbers at the front or head of the date.  Currently only supports dates in 1900 and 2000.
            Heavily relies on datetime for error checking to see
            if date is actually viable. It follows similar ideas to this post:
            http://stackoverflow.com/questions/9978534/match-dates-using-python-regular-expressions

        :param name: str, string that represents a possible name of an object
        :return: datetime.datetime, datetime object with current time or None if not found
        """
        time_formats = ['%Y-%m-%d_%H-%M-%S',
                        '%Y-%m-%d-%H-%M-%S',
                        '%Y-%m-%d--%H-%M-%S',
                        '%y_%m_%dT%H_%M_%S',
                        '%Y-%m-%d%H-%M-%S',
                        '%Y%m%d-%H%M%S',
                        '%Y%m%d-%H%M',
                        '%Y-%m-%d',
                        '%Y%m%d',
                        '%m_%d_%Y',
                        '%m_%d_%y',
                        '%m%d%y',
                        '%m%d%Y',
                        '%d_%m_%Y',
                        '%Y',
                        '%m-%d-%yy',
                        '%m%d%Y']

        mapping = [('%yy', '(([01]\d{1}))'), ('%Y', '((19|20)\d{2})'), ('%y', '(\d{2})'), ('%d', '(\d{2})'),
                   ('%m', '(\d{2})'),
                   ('%H', '(\d{2})'), ('%M', '(\d{2})'), ('%S', '(\d{2})')]
        time_regexes = []
        for time_format in time_formats:
            for k, v in mapping:
                time_format = time_format.replace(k, v)
            time_regexes.append(time_format)

        for time_regex, time_format in zip(time_regexes, time_formats):
            match = cls._get_regex_search(name,
                                          cls.REGEX_DATE % time_regex,
                                          metadata={'format': time_format},
                                          match_index=0)
            if match:
                try:
                    match.update({
                        'datetime': datetime.datetime.strptime(match['match'], time_format.replace('%yy', '%y'))
                    })
                    return match
                except ValueError:
                    pass
        return None

    @classmethod
    def get_string_camel_patterns(cls, name, min_length=0):
        """ Finds all permutations of possible camel casing of the given name

        :param name: str, the name we need to get all possible permutations and abbreviations for
        :param min_length: int, minimum length we want for abbreviations
        :return: list(list(str)), list casing permutations of list of abbreviations
        """
        # Have to check for longest first and remove duplicates
        patterns = []
        abbreviations = list(set(cls._get_abbreviations(name, output_length=min_length)))
        abbreviations.sort(key=len, reverse=True)

        for abbr in abbreviations:
            # We won't check for abbreviations that are stupid eg something with apparent camel casing within
            # the word itself like LeF, sorting from:
            # http://stackoverflow.com/questions/13954841/python-sort-upper-case-and-lower-case
            casing_permutations = list(set(cls._get_casing_permutations(abbr)))
            casing_permutations.sort(key=lambda v: (v.upper(), v[0].islower(), len(v)))
            permutations = [permutation for permutation in casing_permutations if
                            cls.is_valid_camel(permutation) or len(permutation) <= 2]
            if permutations:
                patterns.append(permutations)

        return patterns

    @classmethod
    def _reduce_name(cls, name, parse_dict):
        """ Reduces a name against matches found in a parse dictionary

        :param name: str, name to be reduced
        :param parse_dict: dict, dictionary of matches to reduce against
        :return: str, reduced string
        """
        # Now remove all found entries to make basename regex have an easier time
        removal_indices = []
        for section, match in iteritems(parse_dict):
            try:
                matches = []
                if isinstance(match, dict) and 'compound_matches' in match:
                    matches = match.get('compound_matches')
                elif not isinstance(match, list) and match is not None:
                    matches = [match]

                for m in matches:
                    valid_slice = True
                    slice_a, slice_b = m.get('position')
                    # Adjust slice positions from previous slices
                    if removal_indices is []:
                        removal_indices.append((slice_a, slice_b))

                    for r_slice_a, r_slice_b in removal_indices:
                        if slice_a == r_slice_a and slice_b == r_slice_b:
                            valid_slice = False
                        if slice_a > r_slice_a or slice_a > r_slice_b or slice_b > r_slice_b or slice_b > r_slice_a:
                            slice_delta = r_slice_b - r_slice_a
                            slice_a -= slice_delta
                            slice_b -= slice_delta

                    if valid_slice:
                        name = cls._string_remove_slice(name, slice_a, slice_b)
                        removal_indices.append((slice_a, slice_b))
            except (IndexError, TypeError):
                pass
        return name

    @staticmethod
    def _get_regex_search(input_string, regex, metadata={}, match_index=None, ignore='', flags=0):
        """ Using this so that all results from the functions return similar results

        :param input_string: str, input string to be checked
        :param regex: str, input regex to be compiled and searched with
        :param match_index: (int, None), whether to get a specific match, if None returns all matches as list
        :param metadata: dict, dictionary of extra meta tags needed to identify information
        :return: list(dict), list of dictionaries if multiple hits or a specific entry or None
        """
        generator = re.compile(regex, flags=flags).finditer(input_string)
        matches = []
        for obj in generator:
            try:
                span_a = obj.span(1)
                group_a = obj.group(1)
            except IndexError:
                span_a = obj.span()
                group_a = obj.group()

            if obj.groups() == ('',):
                # Not sure how to account for this situation yet, weird regex.
                return True

            if group_a not in ignore:
                matches.append({'pattern': regex,
                                'input': input_string,
                                'position': span_a,
                                'position_full': obj.span(),
                                'match': group_a,
                                'match_full': obj.group()})

        if matches:
            for match in matches:
                match.update(metadata)
            if match_index is not None:
                return matches[match_index]
            return matches
        return None

    @classmethod
    def _generic_search(cls, name, search_string, metadata={}, ignore=''):
        """ Searches for a specific string given three types of regex search types.  Also auto-checks for camel casing.

        :param name: str, name of object in question
        :param search_string: str, string to find and insert into the search regexes
        :param metadata: dict, metadata to add to the result if we find a match
        :param ignore: str, ignore specific string for the search
        :return: dict, dictionary of search results
        """
        patterns = [cls.REGEX_ABBR_SEOS,
                    cls.REGEX_ABBR_ISLAND,
                    cls.REGEX_ABBR_CAMEL]

        if not search_string[0].isupper():
            patterns.remove(cls.REGEX_ABBR_CAMEL)

        for pattern in patterns:
            search_result = cls._get_regex_search(name,
                                                  pattern.format(ABBR=search_string, SEP=cls.REGEX_SEPARATORS),
                                                  metadata=metadata,
                                                  match_index=0,
                                                  ignore=ignore)
            if search_result is not None:
                if cls.is_valid_camel(search_result.get('match_full'), strcmp=search_result.get('match')):
                    return search_result
        return None

    @staticmethod
    def _get_abbreviations(input_string, output_length=0):
        """ Generates abbreviations for input_string

        :param input_string: str, name of object
        :param output_length: int, optional specific length of abbreviations, default is off
        :return: list(str), list of all combinations that include the first letter (possible abbreviations)
        """
        for i, j in itertools.combinations(range(len(input_string[1:]) + 1), 2):
            abbr = input_string[0] + input_string[1:][i:j]
            if len(abbr) >= output_length:
                yield abbr
            elif output_length == 0:
                yield abbr
        # Have to add the solitary letter as well
        if not output_length or output_length == 1:
            yield input_string[0]

    @classmethod
    def is_valid_camel(cls, input_string, strcmp=None, ignore=''):
        """ Checks to see if an input string is valid for use in camel casing
            This assumes that all lowercase strings are not valid camel case situations and no camel string
            can just be a capitalized word.  Took ideas from here:
            http://stackoverflow.com/questions/29916065/how-to-do-camelcase-split-in-python

        :param input_string: str, input word
        :param strcmp: str, force detection on a substring just in case its undetectable
                            (e.g. part of a section of text that's all lowercase)
        :param ignore: str, what kind of string to ignore in the regex search
        :return: bool, whether it is valid or not
        """
        # clear any non chars from the string
        if not input_string:
            return False

        input_string = ''.join([c for c in input_string if c.isalpha()])
        matches = cls._get_regex_search(input_string,
                                        cls.REGEX_CAMEL.format(SEP=cls.REGEX_SEPARATORS),
                                        match_index=0,
                                        ignore=ignore)
        if matches or input_string == strcmp:
            if strcmp:
                index = input_string.find(strcmp) - 1
                is_camel = strcmp[0].isupper() and input_string[index].islower()
                is_input = strcmp == input_string
                is_start = index + 1 == 0
                return is_camel or is_input or is_start
            return True
        elif len(input_string) == 1:
            return True
        return False

    @staticmethod
    def _split_camel(name):
        """ Splits up a camel case name into its constituent components.

        :param name: str, name to split
        :return: list(str), list of components of the camel case string
        """
        return re.sub('(?!^)([A-Z][a-z]+)', r' \1', name).split()

    @classmethod
    def _get_casing_permutations(cls, input_string):
        """ Takes a string and gives all possible permutations of casing for comparative purposes

        :param input_string: str, name of object
        :return: Generator(str), iterator of all possible permutations of casing for the input_string
        """
        if not input_string:
            yield ""
        else:
            first = input_string[:1]
            for sub_casing in cls._get_casing_permutations(input_string[1:]):
                yield first.lower() + sub_casing
                yield first.upper() + sub_casing

    @staticmethod
    def _string_remove_slice(input_str, start, end):
        """ Removes portions of a string

        :param input_str: str, input string
        :param start: int, end search index
        :param end: int, start search index
        :return: str, the cut string
        """
        if 0 <= start < end <= len(input_str):
            return input_str[:start] + input_str[end:]
        return input_str

    @staticmethod
    def _build_abbreviation_regex(input_string):
        """ builds a recursive regex based on an input string to find possible abbreviations more simply.
            e.g. = punct(u(a(t(i(on?)?)?)?)?)?

        :param input_string: str, input string
        :return: str, output regex
        """
        result = '([%s%s]' % (input_string[0].upper(), input_string[0].lower())
        for char in input_string[1:]:
            result += '[%s%s]?' % (char.upper(), char.lower())
        return result + ')'



#!/usr/bin/python
from core.nomenclature import Nomenclate
import sys

def run():
    arg_dict = {}
    for arg in sys.argv:
        arg_split = arg.split('=')
        if len(arg_split) == 2:
            arg_dict[arg_split[0]] = arg_split[1]
    nom = Nomenclate(arg_dict)
    nom.format = 'working_file'
    return nom.get()



#!/usr/bin/env python
import re
from . import errors as exceptions
import nomenclate.settings as settings

MODULE_LOGGER_LEVEL_OVERRIDE = None


class FormatString(object):
    # Is not a hard coded (word) and does not end with any non word characters or capitals (assuming camel)
    # FORMAT_STRING_REGEX = r'(?:(?<=\()[\w]+(?=\)))|([A-Za-z0-9][^A-Z_\W]+)'
    FORMAT_STRING_REGEX = r'(?:\([\w]+\))|([A-Za-z0-9][^A-Z_\W]+)'
    SEPARATORS = '\\._-?()'

    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)

    def __init__(self, format_string=""):
        self.LOG.info('Initializing format string with input %r' % format_string)
        self.processed_format_order = []
        self.format_string = format_string
        self.format_order = format_string
        self.swap_format(format_string)

    def swap_format(self, format_target):
        try:
            self.get_valid_format_order(format_target)
            self.format_order = format_target
            self.format_string = format_target
            self.LOG.debug('Successfully set format string: %s and format order: %s' % (self.format_string,
                                                                                        self.format_order))
        except exceptions.FormatError as e:
            msg = "Could not validate input format target %s"
            self.LOG.error(msg)
            exceptions.FormatError(msg)

    def parse_format_order(self, format_target):
        """ Dissects the format string and gets the order of the tokens as it finds them l->r
            Splits on camel case or periods/underscores
            Modified version from this:
            http://stackoverflow.com/questions/2277352/python-split-a-string-at-uppercase-letters

        :param format_target: str, format string we want to swap to
        :return: list(str), list of the matching tokens
        """
        self.LOG.debug('Getting format order from target %s' % repr(format_target))
        try:
            pattern = re.compile(self.FORMAT_STRING_REGEX)
            return [match.group() for match in pattern.finditer(format_target) if None not in match.groups()]
        except TypeError:
            raise exceptions.FormatError('Format string %s is not a valid input type, must be <type str>' %
                                         format_target)

    @property
    def format_order(self):
        return self.processed_format_order

    @format_order.setter
    def format_order(self, format_target):
        if format_target:
            self.processed_format_order = self.get_valid_format_order(format_target,
                                                                      format_order=self.parse_format_order(
                                                                          format_target))
        else:
            self.processed_format_order = []

    def get_valid_format_order(self, format_target, format_order=None):
        """ Checks to see if the target format string follows the proper style
        """
        self.LOG.debug('Validating format target %r and parsing for format order %r.' % (format_target, format_order))
        format_order = format_order or self.parse_format_order(format_target)
        self.LOG.debug('Resulting format order is %s' % format_order)

        for format_str in format_order:
            format_target = re.sub(format_str, '', format_target, count=1)

        format_target = re.sub('\([\w]+\)', '', format_target)
        self.LOG.debug('After processing format_target is %s' % format_target)

        for char in format_target:
            if char not in self.SEPARATORS:
                msg = "You have specified an invalid format string %s." % format_target
                self.LOG.warning(msg)
                raise exceptions.FormatError(msg)
        return format_order

    def __str__(self):
        return str(self.format_string)



#!/usr/bin/env python
from six import iteritems

import nomenclate.settings as settings
from . import configurator as config
from . import errors
from . import tokens
from . import formatter
from . import rendering
from .tools import (
    combine_dicts,
    NomenclateNotifier
)

MODULE_LOGGER_LEVEL_OVERRIDE = None


class Nomenclate(object):
    """This class deals with renaming of objects in an approved pattern
    """
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)
    CONFIG_PATH = ['overall_config']

    NAMING_FORMAT_PATH = ['naming_formats']
    DEFAULT_FORMAT_PATH = NAMING_FORMAT_PATH + ['node', 'default']

    SUFFIXES_PATH = ['suffixes']
    OPTIONS_PATH = ['options']
    SIDE_PATH = OPTIONS_PATH + ['side']

    def __init__(self, input_dict=None, format_string='', config_filepath='env.yml', *args, **kwargs):
        """

        :param input_dict: dict, In case the user just passes a dictionary as the first arg in the init, we will merge it.
        :param format_string: str, input format string
        :param config_filepath: str, filepath, full or relative to a config file
        :param args: dict, any amount of dictionaries desired as input
        :param kwargs: str, kwargs to pass to the nomenclate tokens
        """
        if input_dict is None:
            input_dict = dict()

        self.notifier = NomenclateNotifier(self.__setattr__)
        self.LOG.info('***CREATING NEW NOMENCLATE OBJECT***')
        self.LOG.info('Nomenclate init passed args %s and kwargs %s...processing' % (args, kwargs))

        self.cfg = config.ConfigParse(config_filepath=config_filepath)
        self.format_string_object = formatter.FormatString(format_string=format_string)

        self.CONFIG_OPTIONS = dict()

        self.reset_from_config(format_target=format_string)
        self.token_dict = tokens.TokenAttrDictHandler(self)

        self.merge_dict(input_dict, *args, **kwargs)
        self._update_tokens()

    @property
    def empty_tokens(self):
        return {token.token: token.label for token in self.token_dict.unset_token_attrs}

    @property
    def tokens(self):
        return list(self.token_dict.state)

    @property
    def format_order(self):
        return self.format_string_object.format_order

    @property
    def state(self):
        return self.token_dict.state

    @state.setter
    def state(self, input_object):
        """ Property setter used to integrate new settings

        :param input_object: (dict, Nomenclate): input dictionary/instance to integrate
        :return: None
        """
        self.merge_dict(input_object)

    @property
    def format(self):
        return str(self.format_string_object)

    @format.setter
    def format(self, format_target, remove_obsolete_tokens=True):
        """ Changes the internal self.format_string_object format target based on input.  Also checks to see if input
            is an entry in the config file in case we want to switch to a preexisting config format.

        :param format_target: str, input for the new format type.  All strings will be the new tokens.
        :param remove_obsolete_tokens: bool, dictates whether we are removing the obselete tokens or not that
               previously existed
        :return: None
        """
        self.LOG.info('Attempting to swap format to target %r' % format_target)
        original_format, original_format_order = (self.format, self.format_order)

        try:
            self.LOG.info('Looking in config for format target: %r' % format_target)
            format_target = self.cfg.get(format_target, return_type=str, throw_null_return_error=True)
            self.LOG.info('Found entry: %r' % format_target)
        except errors.ResourceNotFoundError:
            pass

        self.LOG.info('Format target not found in config, validating as a format string...')
        self.format_string_object.swap_format(format_target)
        self._update_tokens_from_swap_format(original_format,
                                             original_format_order,
                                             remove_obsolete_tokens=remove_obsolete_tokens)

    def reset_from_config(self, format_target=''):
        self.LOG.info('Starting reset')
        self.initialize_config_settings()
        self.initialize_format_options(format_target=format_target)
        self.initialize_options()
        self.initialize_ui_options()

    def initialize_config_settings(self, input_dict=None):
        self.LOG.info('\tinitialize_config_settings with dict %s' % input_dict)
        input_dict = self.cfg.get(self.CONFIG_PATH, return_type=dict) if input_dict is None else input_dict
        for setting, value in iteritems(input_dict):
            setattr(self, setting, value)
            self.LOG.info('\tNew config value <%s>.%s=%r' % (self.__class__.__name__, setting, getattr(self, setting)))

    def initialize_format_options(self, format_target=''):
        """ First attempts to use format_target as a config path or gets the default format
            if it's invalid or is empty.

        :param format_target: (str, list(str)), can be either a query path to a format
                                                or in format of a naming string the sections should be spaced around
                                                e.g. - this_is_a_naming_string
        :raises: IOError
        """
        self.LOG.info('\tinitialize_format_options: format target=%r' % format_target)
        try:
            if format_target:
                self.format = format_target
                self.LOG.info('\t\tSuccessfully set format target.')
            else:
                raise errors.FormatError
        except errors.FormatError:
            format_target = self.cfg.get(self.DEFAULT_FORMAT_PATH, return_type=str)
            self.LOG.info('\t\tFormat not found, replacing with default format from config path %s: %s' %
                          (self.DEFAULT_FORMAT_PATH, format_target))
            self.format_string_object.swap_format(format_target)

        self.LOG.info('\t\tFormat target is now %s after looking in the config for a match' % format_target)

    def initialize_options(self):
        """ Stores options from the config file

        """
        self.LOG.info('\tinitialize_options')
        self.CONFIG_OPTIONS = self.cfg.get(self.CONFIG_PATH, return_type=dict)

    def initialize_ui_options(self):
        """ Placeholder for all categories/sub-lists within options to be recorded here

        """
        self.LOG.info('\tinitialize_ui_options')
        return self.format

    def get(self, **kwargs):
        """Gets the string of the current name of the object
        Returns (string): the name of the object
        """
        self.LOG.info('RENDERING NOMENCLATE OBJECT')
        self.LOG.info('STATE IS: %s' % self.state)
        self.merge_dict(**kwargs)
        self.LOG.info('STATE IS: %s' % self.state)
        result = rendering.InputRenderer.render_nomenclative(self)
        return result

    def get_chain(self, end, start=None, **kwargs):
        """ Returns a list of names based on index values

        :param end: int, integer for end of sequence
        :param start: (int, optional definition of start position
        :return: list(str), generated object names
        """
        # TODO: rework this entire function, very dirty function.
        """
        var_attr = self.token_dict.get_token_attr(self.VARIATION_PATH[-1])
        var_orig = var_attr.get()

        var_start, n_type = self._get_alphanumeric_index(var_orig)

        if start is None:
            start = var_start
        names = []
        for index in range(start, end + 1):
            if n_type in ['char_hi', 'char_lo']:
                capital = True if n_type == 'char_hi' else False
                var_attr.set(self._get_variation_id(index, capital))

            else:
                var_attr.set(str(index))
            if 'var' in kwargs:
                kwargs.pop("var", None)
            names.append(self.get(**kwargs))
        var_attr.set(var_orig)
        return names
        """
        raise NotImplementedError

    def merge_dict(self, *args, **kwargs):
        """ Takes variable inputs, compiles them into a dictionary then merges it to the current nomenclate's state

        :param args: (dict, Nomenclate), any number of dictionary inputs or Nomenclates to be converted to dicts
        :param kwargs: str, any number of kwargs that represent token:value pairs
        """
        input_dict = self._convert_input(*args, **kwargs)
        self._sift_and_init_configs(input_dict)
        self.LOG.info('Done sifting, now input is %s' % input_dict)
        self.LOG.info('Starting merge, state is %s' % self.token_dict.state)
        self.token_dict.state = input_dict
        self.LOG.info('Finished merge, state is %s' % self.token_dict.state)

    def get_token_settings(self, token, default=None):
        """ Get the value for a specific token as a dictionary or replace with default

        :param token: str, token to query the nomenclate for
        :param default: object, substitution if the token is not found
        :return: (dict, object, None), token setting dictionary or default
        """
        setting_dict = {}

        for key, value in iteritems(self.__dict__):
            if '%s_' % token in key and not callable(key) and not isinstance(value, tokens.TokenAttr):
                setting_dict[key] = self.__dict__.get(key, default)
        return setting_dict

    def _update_tokens(self, specific_tokens=None):
        """ Synchronizes all nomenclate token attributes with values from the token_dict

        :param specific_tokens: list(str): list of tokens to update, otherwise updates all tokens
        """
        self.LOG.info('Force Updating all TokenAttrs')
        if hasattr(self, 'token_dict'):
            specific_tokens = specific_tokens if specific_tokens is not None else list(self.token_dict.state)

            self.LOG.info('Updating nomenclate with %s' % specific_tokens)
            state = {token: getattr(self.token_dict, token) for token in specific_tokens}

            self.LOG.info('Updating nomenclate attributes with state %s' % state)

            for token, token_attr_instance in iteritems(state):
                setattr(self, token, token_attr_instance)

            self.LOG.info('Finished updating Nomenclate object and token_dict.')

    def _purge_tokens(self, token_attrs):
        """ Removes specified token attrs from the instance and from the instance's token_dict to keep synchronized

        :param token_attrs: list(str), list of token attributes to remove
        :return: None
        """
        self.token_dict.purge_tokens(token_attrs)

        for token_attr in token_attrs:
            self.LOG.info('Purging nomenclate attribute - %s' % str(token_attr))
            delattr(self, token_attr)

        self.LOG.info('Finished purging Nomenclate object and token_dict.')

    def _update_tokens_from_swap_format(self, original_format, original_format_order, remove_obsolete_tokens=True):
        """ Updates tokens based on a swap format call that will maintain synchronicity between token_dict and attrs
            If there was an accidental setting already set to one of the attrs that should now be a token attr due
            to the format swap, we wipe it and add a new TokenAttr to the Nomenclate attribute.

        :param original_format: str, original format string to compare to
        :param original_format_order: list(str), the original format order to compare to
        :param remove_obsolete_tokens: bool, whether to remove obsolete tokens
                                             if off: persistent state across format swaps of missing tokens
        """
        old_format_order = [_.lower() for _ in original_format_order]
        new_format_order = [_.lower() for _ in self.format_order]
        if hasattr(self, 'token_dict') and self.format != original_format:
            self.LOG.info('Comparing new format order %s with old format order %s' % (new_format_order,
                                                                                      old_format_order))

            old_tokens = [token for token in list(set(old_format_order) - set(new_format_order))
                          if hasattr(self, token)]

            new_tokens = [token for token in set(new_format_order) - set(old_format_order)
                          if not hasattr(self, token) or isinstance(getattr(self, token, ''), str)]

            self.LOG.info('\nToken Status:\n\tObselete tokens: %s\n\tNew tokens: %s' % (old_tokens, new_tokens))

            self.merge_dict(dict.fromkeys(new_tokens, ''))
            if remove_obsolete_tokens:
                self._purge_tokens(old_tokens)
        else:
            self.LOG.info('No change necessary to update internal token set')

    def _convert_input(self, *args, **kwargs):
        """ Takes variable inputs

        :param args: (dict, Nomenclate), any number of dictionary inputs or Nomenclates to be converted to dicts
        :param kwargs: str, any number of kwargs that represent token:value pairs
        :return: dict, combined dictionary of all inputs
        """
        self.LOG.info('Converting input args %s and kwargs %s' % (args, kwargs))
        args = [arg.state if isinstance(arg, Nomenclate) else arg for arg in args]
        self.LOG.debug('Args after Nomenclate conversion: %s' % args)
        input_dict = combine_dicts(*args, **kwargs)
        self.LOG.info('Converted to %s' % input_dict)
        return input_dict

    def _sift_and_init_configs(self, input_dict):
        """ Removes all key/v for keys that exist in the overall config and activates them.
            Used to weed out config keys from tokens in a given input.
        """
        self.LOG.info('_sift_and_init_configs() - removing config settings from input %s' % input_dict)
        configs = {}
        for k, v in iteritems(input_dict):
            if (k not in map(str.lower, self.format_order) and
                    any([f_order.lower() in k for f_order in self.format_order])):
                try:
                    self.cfg.get(self.CONFIG_PATH + [k])
                except errors.ResourceNotFoundError:
                    pass
                finally:
                    configs[k] = v

        for key, val in iteritems(configs):
            self.LOG.info('Sifting out found config setting %s:%s' % (key, val))
            input_dict.pop(key, None)

        self.initialize_config_settings(input_dict=configs)

    def __eq__(self, other):
        return self.token_dict == other.token_dict

    def __str__(self):
        return self.get()

    def __setattr__(self, key, value):
        """ Custom setattr to detect whether they are trying to set a token, then updating the token_dict

        """
        self.LOG.info('<%s>.__setattr__(%r, %r)' % (self.__class__.__name__, key, value))
        token_dict_attr = getattr(getattr(self, 'token_dict', None), key, None)
        is_not_value_token_attr = not isinstance(value, tokens.TokenAttr)
        is_token_dict_attr_token_attr = isinstance(token_dict_attr, tokens.TokenAttr)

        if all([hasattr(self, 'token_dict'), is_not_value_token_attr, is_token_dict_attr_token_attr]):
            self.LOG.debug('User setting TokenAttr %r -> %r' % (token_dict_attr, value))
            self.token_dict.set_token_attr(key, value)
            # object.__setattr__(self, key, token_dict_attr)
        else:
            self.LOG.debug('User setting attribute %s to %r' % (key, value))
            object.__setattr__(self, key, value)

    def __dir__(self):
        """ Taken from:
            http://techqa.info/programming/question/15507848/the-correct-way-to-override-the-__dir__-method-in-python

        """

        def get_attrs(obj):
            return obj.__dict__.keys()

        def dir_augment(obj):
            attrs = set()
            if not hasattr(obj, '__bases__'):
                # obj is an instance
                instance_class = obj.__class__
                attrs.update(get_attrs(instance_class))
            else:
                # obj is a class
                instance_class = obj

            for cls in instance_class.__bases__:
                attrs.update(get_attrs(cls))
                attrs.update(dir_augment(cls))
            attrs.update(get_attrs(obj))
            return list(attrs)

        return dir_augment(self)



from .nlog import (
    getLogger,
    DEBUG,
    INFO,
    CRITICAL,
    WARNING,
    ERROR,
    FATAL
)

PACKAGE_LOGGER_LEVEL = CRITICAL

CRITICAL = CRITICAL
INFO = INFO
DEBUG = DEBUG
WARNING = WARNING
ERROR = ERROR
FATAL = FATAL


def get_module_logger(module_name, module_override_level=None):
    module_logger_level = module_override_level or PACKAGE_LOGGER_LEVEL
    return getLogger(module_name, level=module_logger_level)



import nomenclate.core.formatter as formatter
import nomenclate.core.errors as exceptions
from . import basetest


class TestFormatStringBase(basetest.TestBase):
    def setUp(self):
        super(TestFormatStringBase, self).setUp()
        self.fs = formatter.FormatString()
        self.fixtures.append(self.fs)


class TestFormatStringValidateFormatString(TestFormatStringBase):
    def test_get__validate_format_string_valid(self):
        self.fs.get_valid_format_order('side_mide')

    def test_get__validate_format_string__is_format_invalid(self):
        print(repr(self))
        self.assertRaises(exceptions.FormatError, self.fs.get_valid_format_order, 'notside;blah')



""" Overall module nomenclate
    It contains a convenience variable Nom which you can instantiate a
    nomenclate.core.nomenclature.Nomenclate object from.
    It also imports the version as __version__ from nomenclate.version for convenience

"""
from . import settings
from . import core
from . import version

__version__ = version.__version__
Nom = core.nomenclature.Nomenclate



from PyQt5.QtWidgets import QApplication, QWidget
from PyQt5 import QtCore, QtGui

QApplication.setApplicationName('progname')
QApplication.setApplicationVersion('0.1')

name = QtGui.qApp.applicationName()
version = QtGui.qApp.applicationVersion()



#!/usr/bin/env python
from six import add_metaclass, iteritems
import re
import string
import datetime
import dateutil.parser as p
from . import errors as exceptions
import nomenclate.settings as settings

try:
    import __builtin__
except ImportError:
    import builtins as __builtin__

from .tools import (
    gen_dict_key_matches,
    flatten
)

MODULE_LOGGER_LEVEL_OVERRIDE = None


class TokenMatch(object):
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)

    def __init__(self, regex_match, substitution, group_name='token'):
        self.match = regex_match.group(group_name)
        self.start = regex_match.start(group_name)
        self.end = regex_match.end(group_name)
        self.sub = str(substitution)

    @property
    def span(self):
        return self.end - self.start

    def adjust_position(self, other, adjust_by_sub_delta=True):
        self._validate_adjuster(other)
        adjustment = other.span - len(other.sub) if adjust_by_sub_delta else other.span
        if adjustment:
            self.LOG.debug('Adjusting %s by %d' % (self, adjustment))
            self._adjust_order(adjustment)

    def _validate_adjuster(self, other):
        if not isinstance(other, self.__class__):
            raise IOError('Only TokenMatch objects are valid inputs to adjust_position')

        other.overlaps(self)

        if self < other:
            raise IndexError('Current TokenMatch is not affected by input TokenMatch\n\t%s\n\t%s' % (repr(self),
                                                                                                     repr(other)))

    def _adjust_order(self, adjust_value):
        self.start -= adjust_value
        self.end -= adjust_value
        self.LOG.debug('New positions are (%d-%d)' % (self.start, self.end))

    def overlaps(self, other):
        if self in other or other in self:
            raise exceptions.OverlapError('Match %s overlaps with an existing match:\n\t%s\n\t%s' % (self.match,
                                                                                                     self, other))
        return True

    def __contains__(self, other):
        try:
            return self.start < other.start < self.end or self.start < other.end < self.end
        except:
            raise NotImplementedError(
                '<%s>.__contains__ does not handle <%s>' % (self.__class__.__name__, type(other)))

    def __eq__(self, other):
        try:
            return (self.start == other.start and self.end == other.end and
                    self.match == other.match and self.sub == other.sub)
        except:
            raise NotImplementedError(
                '<%s>.__eq__ does not handle <%s>' % (self.__class__.__name__, type(other)))

    def __lt__(self, other):
        if isinstance(other, self.__class__):
            return self.end <= other.start
        else:
            raise NotImplementedError

    def __gt__(self, other):
        if isinstance(other, self.__class__):
            return self.start >= other.end
        else:
            raise NotImplementedError

    def __repr__(self):
        return '<%s %s (%d)- [%d:%d] - replacement = %s>' % (self.__class__.__name__,
                                                             self.match, self.span, self.start, self.end, self.sub)

    def __str__(self):
        return '%s:%s' % (self.match, self.sub)


class Nomenclative(object):
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)

    def __init__(self, input_str):
        self.str = input_str
        self.token_matches = []

    def process_matches(self):
        build_str = self.str
        for token_match in self.token_matches:
            if token_match.match == build_str[token_match.start:token_match.end]:
                self.LOG.debug('Processing: %s - %s - %s\n\t%s' % (token_match.match,
                                                                   token_match.sub,
                                                                   token_match.sub,
                                                                   build_str))

                build_str = build_str[:token_match.start] + token_match.sub + build_str[token_match.end:]
                self.adjust_other_matches(token_match)
                self.LOG.debug('Processed as:\n\t%s' % build_str)
        return build_str

    def adjust_other_matches(self, adjuster_match):
        for token_match in [token_match for token_match in self.token_matches if token_match != adjuster_match]:
            try:
                token_match.adjust_position(adjuster_match)
            except IndexError:
                pass
        adjuster_match.end = adjuster_match.start + len(adjuster_match.sub)
        adjuster_match.match = adjuster_match.sub

    def add_match(self, regex_match, substitution):
        token_match = TokenMatch(regex_match, substitution)
        try:
            self.validate_match(token_match)
            self.token_matches.append(token_match)
            self.LOG.info('Added match %s' % self.token_matches[-1])
        except (IndexError, exceptions.OverlapError):
            msg = 'Not adding match %s as it conflicts with a preexisting match' % token_match
            self.LOG.warning(msg)
            raise exceptions.OverlapError(msg)

    def validate_match(self, token_match_candidate):
        for token_match in self.token_matches:
            try:
                token_match.overlaps(token_match_candidate)
            except exceptions.OverlapError:
                msg = "Cannot add match %s due to overlap with %s" % (token_match, token_match_candidate)
                self.LOG.error(msg)
                raise exceptions.OverlapError(msg)

    def __str__(self):
        matches = '' if not self.token_matches else '\n'.join(map(str, self.token_matches))
        return '%s:%s' % (self.str, matches)


class InputRenderer(type):
    RENDER_FUNCTIONS = {}
    REGEX_PARENTHESIS = r'([\(\)]+)'
    REGEX_BRACKETS = r'([\{\}]+)'
    REGEX_STATIC_TOKEN = r'(\(\w+\))'
    REGEX_BRACKET_TOKEN = r'(\{\w+\})'
    REGEX_TOKEN_SEARCH = r'(?P<token>((?<![a-z]){TOKEN}(?![0-9]))|((?<=[a-z]){TOKEN_CAPITALIZED}(?![0-9])))'

    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)

    def __new__(mcs, name, bases, dct):
        cls = type.__new__(mcs, name, bases, dct)
        token = dct.get('token', None)
        if token:
            mcs.RENDER_FUNCTIONS[token] = cls
        return cls

    @classmethod
    def render_unique_tokens(cls, nomenclate_object, input_dict):
        cls.LOG.info('Current list of render functions: %s' % list(cls.RENDER_FUNCTIONS))
        cls.LOG.info('Checking against input dictionary %s' % input_dict)
        non_empty_token_entries = {_k: _v for _k, _v in iteritems(input_dict) if _v}

        for token, value in iteritems(non_empty_token_entries):
            cls.LOG.info('Checking for unique token on token %s:%r' % (token, value))
            for func in cls.get_valid_render_functions(token):
                renderer = cls.RENDER_FUNCTIONS.get(func, None)
                cls.LOG.info('Finding token specific render function for token %r with renderer %s' % (token, renderer))

                if func == 'default':
                    renderer.token = token

                if callable(getattr(renderer, 'render')):
                    cls.LOG.info('render_unique_tokens() - Rendering token %r: %r, token settings=%s' %
                                 (token, value, nomenclate_object.get_token_settings(token)))

                    rendered_token = renderer.render(value, token, nomenclate_object,
                                                     **nomenclate_object.get_token_settings(token))
                    cls.LOG.info('Unique token %s rendered as: %s' % (token, rendered_token))

                    input_dict[token] = rendered_token

    @classmethod
    def get_valid_render_functions(cls, token_name):
        render_functions = []
        for func in list(cls.RENDER_FUNCTIONS):
            is_sub_token = token_name.replace(func, '').isdigit()
            is_token_renderer = not token_name.replace(func, '')
            if is_sub_token or is_token_renderer:
                render_functions.append(func)
        cls.LOG.info('Found valid render functions for token %s: %s' % (token_name, render_functions))
        return render_functions or ['default']

    @classmethod
    def render_nomenclative(cls, nomenclate_object):
        nomenclative = Nomenclative(nomenclate_object.format)
        token_values = nomenclate_object.token_dict.token_attr_dict
        cls.LOG.info('render_nomenclative() - Current state is %s with nomenclative %s' % (token_values, nomenclative))
        cls.render_unique_tokens(nomenclate_object, token_values)
        rendered_nomenclative = nomenclate_object.format
        cls.LOG.info('Finished rendering unique tokens.')
        cls._prepend_token_match_objects(token_values, rendered_nomenclative)

        for token, match_value in iteritems(token_values):
            nomenclative.add_match(*match_value)

        cls.LOG.info('Before processing state has been updated to %s' % nomenclative)
        rendered_nomenclative = cls.cleanup_formatted_string(nomenclative.process_matches())
        cls.LOG.info('Finally converted to %s' % rendered_nomenclative)
        return rendered_nomenclative

    @classmethod
    def _prepend_token_match_objects(cls, token_values, incomplete_nomenclative):
        for token, value in iteritems(token_values):
            re_token = cls.REGEX_TOKEN_SEARCH.format(TOKEN=token,
                                                     TOKEN_CAPITALIZED=token[0].upper() + token[1:])
            re_matches = re.finditer(re_token, incomplete_nomenclative, 0)

            for re_match in re_matches:
                token_values[token] = (re_match, value)

        cls._clear_non_matches(token_values)

    @staticmethod
    def _clear_non_matches(token_values):
        to_delete = []
        for token, value in iteritems(token_values):
            if isinstance(value, str) or not isinstance(value, tuple):
                to_delete.append(token)

        for delete in to_delete:
            token_values.pop(delete)

    @classmethod
    def cleanup_formatted_string(cls, formatted_string):
        """ Removes unused tokens/removes surrounding and double underscores

        :param formatted_string: str, string that has had tokens replaced
        :return: str, cleaned up name of object
        """
        # TODO: chunk this out to sub-processes for easier error checking, could be own class
        # Remove whitespace
        result = formatted_string.replace(' ', '')
        # Remove any static token parentheses
        result = re.sub(cls.REGEX_PARENTHESIS, '', result)
        # Remove any multiple underscores
        result = re.sub('_+', '_', result)
        # Remove trailing or preceding non letter characters
        result = re.sub(r'(^[\W_]+)|([\W_]+$)', '', result)
        #  not sure what this one was...but certainly not it.
        result = re.sub(r'(\()|(\))', '', result)
        return result

    @staticmethod
    def _get_alphanumeric_index(query_string):
        """ Given an input string of either int or char, returns what index in the alphabet and case it is

        :param query_string: str, query string
        :return: (int, str), list of the index and type
        """
        # TODO: could probably rework this. it works, but it's ugly as hell.
        try:
            return [int(query_string), 'int']
        except ValueError:
            if len(query_string) == 1:
                if query_string.isupper():
                    return [string.ascii_uppercase.index(query_string), 'char_hi']
                elif query_string.islower():
                    return [string.ascii_lowercase.index(query_string), 'char_lo']
            else:
                raise IOError('The input is a string longer than one character')


@add_metaclass(InputRenderer)
class RenderBase(object):
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)
    token = 'default'

    @classmethod
    def render(cls,
               value,
               token,
               nomenclate_object,
               config_query_path=None,
               return_type=list,
               use_value_in_query_path=True,
               **filter_kwargs):
        """ Default renderer for a token.  It checks the config for a match, if not found it uses the value provided.

        :param value: str, value we are trying to match (or config setting for the token)
        :param token: str, token we are searching for
        :param nomenclate_object: nomenclate.core.nomenclate.Nomenclate, instance of nomenclate object to query
        :param filter_kwargs: any config settings that relate to the token as found from the nomenclate instance
        :return: str, the resulting syntactically rendered string
        """
        if config_query_path == None:
            config_query_path = nomenclate_object.OPTIONS_PATH + [token]
            if use_value_in_query_path:
                config_query_path += [value]

        cls.LOG.info('Attempting to default render %s with value %s and kwargs %s' % (token, value, filter_kwargs))
        config_matches = cls.get_config_match(value,
                                              token,
                                              config_query_path,
                                              return_type,
                                              nomenclate_object,
                                              **filter_kwargs)

        options = cls.flatten_input(config_matches, value)
        option = cls.process_criteria(token, options, **filter_kwargs) if options else value
        return cls.process_token_augmentations(option, token_attr=getattr(nomenclate_object, token))

    @classmethod
    def process_token_augmentations(cls, value, token_attr):
        """ Uses any found augmentations from the TokenAttr to augment the final rendered value.  Currently
            this only processes the augmentations:
                TokenAttr().case
                TokenAttr().prefix
                TokenAttr().suffix

        :param value: str, the resulting rendered string from the TokenAttr
        :param token_attr: nomenclate.core.tokens.TokenAttr, the processed TokenAttr to be used to query settings.
        :return: str, final augmented string
        """
        cls.LOG.info('Processing augmentations for token attr %r and applying to value %r' % (token_attr, value))
        value = str(value)
        case_value = getattr(token_attr, 'case')

        value = getattr(value, case_value, str)() or value
        value = getattr(token_attr, 'prefix') + value
        value = value + getattr(token_attr, 'suffix')

        return value

    @classmethod
    def get_config_match(cls,
                         query_string,
                         token,
                         entry_path,
                         return_type,
                         nomenclate_object,
                         **filter_kwargs):
        """ Queries the nomenclate's config data for corresponding entries and filters against the incoming
            filter_kwargs as detailed in cls.process_criteria

        :param query_string: str, string to look for once we find a match in the config
        :param token: str, the given token to search for
        :param entry_path: list(str), the query path to the subsection we are looking for to get a match in the config
        :param return_type: type, the type of return value we want (usually should be <type str>)
        :param nomenclate_object: nomenclate.core.nomenclature.Nomenclate, instance to query against (has config data)
        :return: object, whatever return type was specified
        """
        try:
            return nomenclate_object.cfg.get(entry_path, return_type=return_type)
        except exceptions.ResourceNotFoundError:
            cls.LOG.warning('No entry for token %s - defaulting to current: %s' % (token, query_string))
            return query_string

    @classmethod
    def flatten_input(cls, options, query_string):
        """ Takes a list, dict or str of options and outputs a filtered list
            Behavior list:
                dict: flattens/filters the dict to only key matches based on query_string
                list: just flattens a list just in case it's nested.
                str: returns a list with the string in it

        :param options: list, dict, str, input options to flatten/filter
        :param query_string: str, string we are looking for if the input is a dictionary
        :return: list, flattened list.
        """
        cls.LOG.info('Flattening input %s for matches with %s' % (options, query_string))
        if not isinstance(options, (dict, list)):
            options = [options]
        else:
            if isinstance(options, dict):
                options = list(gen_dict_key_matches(query_string, options))

            options = list(flatten(options))
        cls.LOG.info('Flattened to %s' % options)
        return options

    @classmethod
    def process_criteria(cls, token, options, **filter_kwargs):
        """ Each kwarg passed is considered a filter.  The kwarg is in format <token>_<filter function> and if the
            filter function is found in __builtins__ it uses the filter function and checks the result against
            the kwarg's value.  If it passes the check it is filtered out of the current list of options

        :param token: str, token we are querying
        :param options: list(str), the options to filter with kwargs
        :param filter_kwargs: dict(str: str), dictionary of {<token>_<__builtin__ function>: compare value}
        :return:
        """
        options = [options] if isinstance(options, str) else options
        criteria_matches = list(options)

        for token_criteria, criteria_value in iteritems(filter_kwargs):
            builtin_func = token_criteria.replace('%s_' % token, '')
            try:
                builtin_func = getattr(__builtin__, builtin_func)
                cls.LOG.info('Filtering options: %s with criteria: %s' % (options, token_criteria))

                for option in options:
                    cls.LOG.info('Running through option %s' % option)
                    if token_criteria and criteria_value:
                        if not builtin_func(option) == criteria_value:
                            cls.LOG.info('Non criteria match for %s(%s)=%s' % (builtin_func, option, criteria_value))
                            criteria_matches.remove(option)
            except AttributeError:
                cls.LOG.warning('Criteria function %r is invalid...skipping' % builtin_func)
        cls.LOG.info('Found criteria matches: %s ...returning first' % criteria_matches)

        return criteria_matches[0]


class RenderDate(RenderBase):
    token = 'date'

    @classmethod
    def render(cls, date, token, nomenclate_object, **filter_kwargs):
        if date == 'now':
            d = datetime.datetime.now()
        else:
            try:
                d = p.parse(date)
            except ValueError:
                return ''
        date_format = getattr(nomenclate_object, '%s_format' % cls.token, '%Y-%m-%d')
        return d.strftime(date_format)


class RenderVar(RenderBase):
    token = 'var'

    @classmethod
    def render(cls, var, token, nomenclate_object, **filter_kwargs):
        if var:
            var_index = filter_kwargs.get('%s_index' % cls.token, 0)
            return cls._get_variation_id(var_index, var.isupper())
        else:
            return var

    @staticmethod
    def _get_variation_id(integer, capital=False):
        """ Convert an integer value to a character. a-z then double aa-zz etc
        Args:
            integer (int): integer index we're looking up
            capital (bool): whether we convert to capitals or not
        Returns (str): alphanumeric representation of the index
        """
        # calculate number of characters required
        base_power = base_start = base_end = 0
        while integer >= base_end:
            base_power += 1
            base_start = base_end
            base_end += pow(26, base_power)
        base_index = integer - base_start

        # create alpha representation
        alphas = ['a'] * base_power
        for index in range(base_power - 1, -1, -1):
            alphas[index] = chr(int(97 + (base_index % 26)))
            base_index /= 26

        characters = ''.join(alphas)
        return characters.upper() if capital else characters


class RenderVersion(RenderBase):
    token = 'version'

    @classmethod
    def render(cls, version, token, nomenclate_object, **filter_kwargs):
        padding = filter_kwargs.get('%s_padding' % token, 4)
        format = filter_kwargs.get('%s_format' % token, '#')
        version_string = format.replace('#', '%0{0}d')
        return version_string.format(padding) % version


class RenderType(RenderBase):
    __metaclass__ = InputRenderer
    token = 'type'

    @classmethod
    def render(cls, engine_type, token, nomenclate_object, **filter_kwargs):
        return super(RenderType, cls).render(engine_type,
                                             cls.token,
                                             nomenclate_object,
                                             config_query_path=nomenclate_object.SUFFIXES_PATH,
                                             return_type=dict,
                                             **filter_kwargs)



""" The core suite of modules for nomenclate's operation

"""
from . import configurator
from . import errors
from . import formatter
from . import nameparser
from . import nomenclature
from . import rendering
from . import tokens
from . import tools



#!/usr/bin/python

if __name__ == '__main__':
    import sys
    import os
    # Add the top directory into sys.path
    sys.path.append(os.path.dirname(os.path.dirname(__file__)))

    from nomenclate import app
    sys.exit(app.run())
